package org.apache.maven;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;
import java.util.List;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.xml.sax.Attributes;
import org.xml.sax.InputSource;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * A class to parse the POM.
 */
public class BootstrapPomParser
    extends DefaultHandler
{
    /** the project dependencies */
    private ArrayList dependencies = new ArrayList();

    /** current dependency being processed */
    private Dependency currentDependency;

    /** factory to create a sax parser */
    private static SAXParserFactory saxFactory;

    /** whether we're inside a dependency element during parsing */
    private boolean insideDependency = false;

    /** text of the element being processed */
    private StringBuffer bodyText = new StringBuffer();

    /**
     * @return the dependencies attribute of the Bootstrap object
     */
    public List getDependencies()
    {
        return dependencies;
    }

    /**
     * Parse a POM
     * @param project the project file to parse
     */
    public void parse( File project )
    {
        try
        {
            saxFactory = SAXParserFactory.newInstance();
            SAXParser parser = saxFactory.newSAXParser();
            InputSource is = new InputSource( new FileInputStream( project ) );
            parser.parse( is, this );
        }
        catch ( Exception e )
        {
            e.printStackTrace();
        }
    }

    /**
     * Handles opening elements of the xml file.
     * @param uri the uri being parsed
     * @param localName element without namespace
     * @param rawName element name
     * @param attributes element attributes
     */
    public void startElement( String uri, String localName, String rawName, Attributes attributes )
    {
        if ( "dependency".equals( rawName ) )
        {
            currentDependency = new Dependency();
            insideDependency = true;
        }
    }

    /**
     * some xml element text
     * @param buffer the text found
     * @param start the start position of text in the buffer
     * @param length the length of the text in the buffer
     */
    public void characters( char[] buffer, int start, int length )
    {
        bodyText.append( buffer, start, length );
    }

    /**
     * @return the current text from the characters method
     */
    private String getBodyText()
    {
        return bodyText.toString().trim();
    }

    /**
     * Handles closing elements of the xml file.
     * @param uri the uri being parsed
     * @param localName element without namespace
     * @param rawName element name
     */
    public void endElement( String uri, String localName, String rawName )
    {
        if ( "dependency".equals( rawName ) )
        {
            dependencies.add( currentDependency );
            insideDependency = false;
        }
        else if ( "id".equals( rawName ) && insideDependency )
        {
            currentDependency.setId( getBodyText() );
        }
        else if ( "version".equals( rawName ) && insideDependency )
        {
            currentDependency.setVersion( getBodyText() );
        }
        else if ( "jar".equals( rawName ) && insideDependency )
        {
            currentDependency.setJar( getBodyText() );
        }
        else if ( "type".equals( rawName ) && insideDependency )
        {
            currentDependency.setType( getBodyText() );
        }
        else if ( "groupId".equals( rawName ) && insideDependency )
        {
            currentDependency.setGroupId( getBodyText() );
        }
        else if ( "artifactId".equals( rawName ) && insideDependency )
        {
            currentDependency.setArtifactId( getBodyText() );
        }

        bodyText = new StringBuffer();
    }

    /**
     * Warning callback.
     *
     * @param spe The parse exception that caused the callback to be invoked.
     */
    public void warning( SAXParseException spe )
    {
        printParseError( "Warning", spe );
    }

    /**
     * Error callback.
     *
     * @param spe The parse exception that caused the callback to be invoked.
     */
    public void error( SAXParseException spe )
    {
        printParseError( "Error", spe );
    }

    /**
     * Fatal error callback.
     *
     * @param spe The parse exception that caused the callback to be invoked.
     */
    public void fatalError( SAXParseException spe )
    {
        printParseError( "Fatal Error", spe );
    }

    /**
     * Description of the Method
     * @param type the type of error
     * @param spe The parse exception that caused the callback to be invoked.
     */
    private final void printParseError( String type, SAXParseException spe )
    {
        System.err.println( type + " [line " + spe.getLineNumber() + ", row " + spe.getColumnNumber() + "]: "
            + spe.getMessage() );
    }
}
