package org.apache.maven;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.StringTokenizer;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.PatternSet;

/**
 * Get a list of resources from a website.
 *
 * @author <a href="mailto:jason@zenplex.com">Jason van Zyl</a>
 */
public class BootstrapTask
    extends Task
{
    /** the parser to load the pom */
    private BootstrapPomParser bootstrapPomParser;

    /** the pom to be bootstrapped */
    private File projectDescriptor;

    /** the dependencies of the pom */
    private List dependencies;

    /** the local repo of jars */
    private File mavenRepoLocal;

    /** whether to use the file timestamp when checking freshness */
    private boolean useTimestamp = true;

    /** whether to ignore errors on downloads */
    private boolean ignoreErrors = true;

    /** the url to download from */
    private String baseUrl;

    /** whether we've got 'net access */
    private String online;

    /** the proxy to use */
    private String proxyHost;

    /** the port to use on the proxy */
    private String proxyPort;

    /** the user name to use for the proxy */
    private String proxyUserName;

    /** the password to use for the proxy */
    private String proxyPassword;

    /** the NTLM login host. */
    private String loginHost = null;

    /** the NTLM login domain. */
    private String loginDomain = null;

    /** list of files to process */
    private List files;

    /** Location to put the cope_deps scripts. */
    private File copyScriptDir = null;

    /** number of times to retry download */
    private static final int MAX_RETRIES = 3;

    /**
     * Get the directory to put the copy script into.
     * @return the directory
     */
    public File getCopyScriptDir()
    {
        return copyScriptDir;
    }

    /**
     * Set the directory to put the copy script into.
     * @param copyScriptDir the copy script directory
     */
    public void setCopyScriptDir( File copyScriptDir )
    {
        this.copyScriptDir = copyScriptDir;
    }

    /**
     * Sets the projectDescriptor attribute of the BootstrapTask object
     * @param projectDescriptor the file to bootstrap
     */
    public void setProjectDescriptor( File projectDescriptor )
    {
        this.projectDescriptor = projectDescriptor;
    }

    /**
     * @return the projectDescriptor attribute
     */
    public File getProjectDescriptor()
    {
        return projectDescriptor;
    }

    /**
     * Sets the mavenRepoLocal attribute of the BootstrapTask object
     * @param mavenRepoLocal the repository to use
     */
    public void setMavenRepoLocal( File mavenRepoLocal )
    {
        this.mavenRepoLocal = mavenRepoLocal;
    }

    /**
     * @return the local maven repository
     */
    public File getMavenRepoLocal()
    {
        return mavenRepoLocal;
    }

    // http

    /**
     * Sets the files attribute of the BootstrapTask object
     * @param files a list of dependency file names
     */
    public void setFiles( List files )
    {
        this.files = files;
    }

    /**
     * @return the files attribute
     */
    public List getFiles()
    {
        return files;
    }

    /**
     * Sets the baseUrl attribute of the Get object
     *
     * @param baseUrl A URL that the files are relative to
     */
    public void setBaseUrl( String baseUrl )
    {
        this.baseUrl = baseUrl;
    }

    /**
     * Sets the proxyHost attribute of the Get object
     *
     * @param proxyHost the host name of the proxy
     */
    public void setProxyHost( String proxyHost )
    {
        if ( validProperty( proxyHost ) )
        {
            this.proxyHost = proxyHost;
        }
    }

    /**
     * Sets the proxyPort attribute of the Get object
     *
     * @param proxyPort the port used to access the proxy, e.g. <code>8080</code>
     */
    public void setProxyPort( String proxyPort )
    {
        if ( validProperty( proxyPort ) )
        {
            this.proxyPort = proxyPort;
        }
    }

    /**
     * Sets the proxyUserName attribute of the Get object
     *
     * @param proxyUserName the login used to access the proxy
     */
    public void setProxyUserName( String proxyUserName )
    {
        if ( validProperty( proxyUserName ) )
        {
            this.proxyUserName = proxyUserName;
        }
    }

    /**
     * Sets the proxyPassword attribute of the Get object
     *
     * @param proxyPassword the password used to access the proxy, e.g. <code>8080</code>
     */
    public void setProxyPassword( String proxyPassword )
    {
        if ( validProperty( proxyPassword ) )
        {
            this.proxyPassword = proxyPassword;
        }
    }

    /**
     * Sets the loginHost attribute of the Get object
     *
     * @param loginHost the host used to access the NTLM proxy from
     */
    public void setLoginHost( String loginHost )
    {
        if ( validProperty( loginHost ) )
        {
            this.loginHost = loginHost;
        }
    }

    /**
     * Sets the loginDomain attribute of the Get object
     *
     * @param loginDomain the domain used to access the NTLM proxy on
     */
    public void setLoginDomain( String loginDomain )
    {
        if ( validProperty( loginDomain ) )
        {
            this.loginDomain = loginDomain;
        }
    }

    /**
     * Don't stop if get fails if set to "<CODE>true</CODE>".
     *
     * @param v if "true" then don't report download errors up to ant
     */
    public void setIgnoreErrors( boolean v )
    {
        ignoreErrors = v;
    }

    /**
     * Use timestamps, if set to "<CODE>true</CODE>". <p>
     *
     * In this situation, the if-modified-since header is set so that the file
     * is only fetched if it is newer than the local file (or there is no local
     * file) This flag is only valid on HTTP connections, it is ignored in other
     * cases. When the flag is set, the local copy of the downloaded file will
     * also have its timestamp set to the remote file time. <br>
     * Note that remote files of date 1/1/1970 (GMT) are treated as 'no
     * timestamp', and web servers often serve files with a timestamp in the
     * future by replacing their timestamp with that of the current time. Also,
     * inter-computer clock differences can cause no end of grief.
     *
     * @param v "true" to enable file time fetching
     */
    public void setUseTimestamp( boolean v )
    {
        useTimestamp = v;
    }

    /**
     * Does the work.
     *
     * @exception BuildException Thrown in unrecoverable error.
     */
    public void execute()
        throws BuildException
    {
        if ( baseUrl == null )
        {
            throw new BuildException( "baseUrl attribute is required" );
        }

        if ( mavenRepoLocal == null )
        {
            throw new BuildException( "mavenRepoLocal attribute is required" );
        }

        if ( mavenRepoLocal.exists() && !mavenRepoLocal.canWrite() )
        {
            throw new BuildException( "Can't write to " + mavenRepoLocal.getAbsolutePath() );
        }

        System.out.println( "Using the following for your maven.repo.local: " + mavenRepoLocal );

        bootstrapPomParser = new BootstrapPomParser();

        bootstrapPomParser.parse( getProjectDescriptor() );

        dependencies = bootstrapPomParser.getDependencies();

        List list = new ArrayList();

        for ( Iterator i = dependencies.iterator(); i.hasNext(); )
        {
            Dependency d = (Dependency) i.next();

            list.add( getArtifactPath( d, "/" ) );
        }

        setFiles( list );
        // if offline is not set, download dependencies
        if ( !"-o".equals( getOnline() ) )
        {
            downloadDependencies();
        }

        createClasspathReference( getProject(), "bootstrap-classpath", list, mavenRepoLocal );

        createPatternSetReference( getProject(), "bootstrap-patternset", list );
    }

    /**
     * Get the artifact path for a dependency in the repository.
     * @param d the dependency
     * @param pathSeparator the path separator (e.g. '/') to use in building the path
     * @return the path to the dependency
     */
    private String getArtifactPath( Dependency d, String pathSeparator )
    {
        return d.getArtifactDirectory() + pathSeparator + "jars" + pathSeparator + d.getArtifact();
    }

    /**
     * Does the work.
     *
     * @exception BuildException Thrown in unrecoverable error.
     */
    public void downloadDependencies()
        throws BuildException
    {
        List repositoryUrls = new ArrayList();
        StringTokenizer tok = new StringTokenizer( baseUrl, "," );
        while ( tok.hasMoreTokens() )
        {
            String url = tok.nextToken().trim();
            if ( !url.endsWith( "/" ) )
            {
                url = url + "/";
            }
            repositoryUrls.add( url );
        }

        try
        {
            ArtifactDownloader downloader = new ArtifactDownloader( mavenRepoLocal.getAbsolutePath(), repositoryUrls );
            downloader.setProxyHost( proxyHost );
            downloader.setProxyPort( proxyPort );
            downloader.setProxyUserName( proxyUserName );
            downloader.setProxyPassword( proxyPassword );
            downloader.downloadDependencies( getFiles() );
        }
        catch ( Exception e )
        {
            throw new BuildException( "Failed to download dependency", e );
        }
    }

    /**
     * Create a classpath reference in <code>project</code>, named
     * <code>reference</code> from the <code>list</code> of jar files, relative
     * to <code>baseDir</code>
     * @param project the Ant project to create the reference in
     * @param reference the name of the reference to create
     * @param list the list of jar files to append to the new classpath
     * reference
     * @param baseDir the directory the specified jars are relative to
     */
    public static void createClasspathReference( Project project, String reference, List list, File baseDir )
    {
        Path classpath = new Path( project );

        for ( Iterator i = list.iterator(); i.hasNext(); )
        {
            String jar = (String) i.next();
            Path p = new Path( project );
            p.setPath( new File( baseDir, jar ).getAbsolutePath() );
            classpath.append( p );
        }

        project.addReference( reference, classpath );
    }

    /**
     * Create a pattern set reference in <code>project</code>, named
     * <code>reference</code> from the <code>list</code> of includes/excludes/
     * file names
     * @param project the Ant project to create the reference in
     * @param reference the name of the reference to create
     * @param list the list of lines (Strings) specifying include=, exclude= or
     *  a file name
     */
    public static void createPatternSetReference( Project project, String reference, List list )
    {
        StringBuffer includesSb = new StringBuffer();
        StringBuffer excludesSb = new StringBuffer();
        PatternSet patternSet = new PatternSet();

        for ( Iterator i = list.iterator(); i.hasNext(); )
        {
            String line = (String) i.next();
            line = line.trim();
            includesSb.append( line ).append( ',' );
        }

        String includes = includesSb.toString();
        String excludes = excludesSb.toString();

        if ( includes.length() > 0 )
        {
            // Remove trailing comma
            includes = includes.substring( 0, includes.length() - 1 );
            patternSet.setIncludes( includes );
        }
        if ( excludes.length() > 0 )
        {
            // Remove trailing comma
            excludes = includes.substring( 0, includes.length() - 1 );
            patternSet.setExcludes( excludes );
        }

        project.addReference( reference, patternSet );
    }

    /**
     * @param property the property to test
     * @return true if the given property is not null and startsWith '${'
     */
    private boolean validProperty( String property )
    {
        if ( ( property != null ) && ( property.length() > 0 ) && !property.startsWith( "${" ) )
        {
            return true;
        }

        return false;
    }

    /**
     * Returns the online.
     * @return String
     */
    public String getOnline()
    {
        return online;
    }

    /**
     * Sets the online.
     * @param online The online to set
     */
    public void setOnline( String online )
    {
        this.online = online;
    }

}
