package org.apache.maven.util;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.security.MessageDigest;

/**
 * Encode an MD5 digest into a String. <p>
 *
 * The 128 bit MD5 hash is converted into a 32 character long String. Each
 * character of the String is the hexadecimal representation of 4 bits of the
 * digest.
 *
 * XXX The API here is a mess.  It is combining a static utility class with a
 *     message digest API. Some methods which should be static are not, presumably
 *     so Jelly can easily access them.
 *
 * @author Remy Maucherat
 * @author <a href="mailto:jason@zenplex.com">Jason van Zyl</a>
 * @version $Revision: 517014 $ $Date: 2007-03-11 22:15:50 +0100 (dim., 11 mars 2007) $
 */
public class MD5Sum
{
    /** File that will be checksummed. */
    private File file;

    /** Checksum */
    private String checksum;

    /** Hex digits. */
    private static final char[] hexadecimal = {
        '0',
        '1',
        '2',
        '3',
        '4',
        '5',
        '6',
        '7',
        '8',
        '9',
        'a',
        'b',
        'c',
        'd',
        'e',
        'f' };

    /** Set the file to perform the checksum on.
     *
     *  @param file The file.
     */
    public void setFile( File file )
    {
        this.file = file;
    }

    /** Get the file that the checksum will be perform on.
     *
     *  @return the file.
     */
    public File getFile()
    {
        return file;
    }

    /** Set the checksum
     *
     *  @param checksum The checksum.
     */
    public void setChecksum( String checksum )
    {
        this.checksum = checksum;
    }

    /** Get the checksum
     *
     *  @return The calculated checksum.
     */
    public String getChecksum()
    {
        return checksum;
    }

    /**
     * Encodes the 128 bit (16 bytes) MD5 into a 32 character String.
     *
     * @param binaryData Array containing the digest
     *
     * @return Encoded MD5, or null if encoding failed
     */
    public String encode( byte[] binaryData )
    {

        if ( binaryData.length != 16 )
        {
            return null;
        }

        char[] buffer = new char[32];

        for ( int i = 0; i < 16; i++ )
        {
            int low = ( binaryData[i] & 0x0f );
            int high = ( ( binaryData[i] & 0xf0 ) >> 4 );
            buffer[i * 2] = hexadecimal[high];
            buffer[i * 2 + 1] = hexadecimal[low];
        }

        return new String( buffer );
    }

    /** Pull in static content and store it
     *
     *  @param file The file to read.
     *
     *  @return The bytes of the file.
     *
     *  @throws Exception If an error occurs reading in the file's bytes.
     */
    public byte[] getBytes( File file )
        throws Exception
    {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        InputStream stream = new FileInputStream( file );

        byte buf[] = new byte[1024];
        int len = 0;

        while ( ( len = stream.read( buf, 0, 1024 ) ) != -1 )
        {
            baos.write( buf, 0, len );
        }

        try
        {
            stream.close();
        }
        catch ( IOException e )
        {
            // Nothing to do
        }

        return baos.toByteArray();
    }

    /** Perform the MD5-Sum work.
     *
     *  @throws Exception If an error occurs while calculating the sum.
     */
    public void execute()
        throws Exception
    {
        if ( !file.exists() )
        {
            System.err.println( "Specified file " + file + " doesn't exist." );
        }

        MessageDigest md5Helper = MessageDigest.getInstance( "MD5" );
        setChecksum( encode( md5Helper.digest( getBytes( file ) ) ) );
    }
}
