package org.apache.maven.werkz;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

/*
 $Id: Session.java 517014 2007-03-11 21:15:50Z ltheussl $

 Copyright 2002 (C) The Werken Company. All Rights Reserved.
 
 Redistribution and use of this software and associated documentation
 ("Software"), with or without modification, are permitted provided
 that the following conditions are met:

 1. Redistributions of source code must retain copyright
 statements and notices.  Redistributions must also contain a
 copy of this document.
 
 2. Redistributions in binary form must reproduce the
 above copyright notice, this list of conditions and the
 following disclaimer in the documentation and/or other
 materials provided with the distribution.
 
 3. The name "werkz" must not be used to endorse or promote
 products derived from this Software without prior written
 permission of The Werken Company.  For written permission,
 please contact bob@werken.com.
 
 4. Products derived from this Software may not be called "werkz"
 nor may "werkz" appear in their names without prior written
 permission of The Werken Company. "werkz" is a registered
 trademark of The Werken Company.
 
 5. Due credit should be given to "the werkz project"
 ( http://werkz.werken.com/ ).
 
 THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.
 
 */

import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;

/** Goal satisfaction state.
 *
 *  <p>
 *  A <code>Session</code> isolates satisfaction of <code>Goal</code>s
 *  into an objectified context.  Multiple <code>Session</code>s can
 *  be deployed simultaneously against a <code>Project</code> or
 *  arbitrary <code>Goal</code> graph.  Likewise, a single <code>Session</code>
 *  may be used simultaneous against several <code>Project</code>s
 *  or <code>Goal</code> graphs.  A caveat with this last idiom
 *  revolves around <code>Goal</code>'s implementation of equality
 *  which utilitizes only the <code>Goal</code>'s name.
 *  </p>
 *  
 *  @see WerkzProject
 *  @see Goal
 *
 *  @author <a href="mailto:bob@eng.werken.com">bob mcwhirter</a>
 */
public class Session
{
    // ------------------------------------------------------------
    //     Instance members
    // ------------------------------------------------------------

    /** Set of <code>Goal</code>s satisfied within this context. */
    private Set satisfiedGoals;

    /** Set of attributes that can be used during the execution of <code>Action</code>s. */
    private Map attributes;

    // ------------------------------------------------------------
    //     Constructors
    // ------------------------------------------------------------

    /** Construct.
     */
    public Session()
    {
        this.satisfiedGoals = new HashSet();
        this.attributes = new HashMap();
    }

    // ------------------------------------------------------------
    //     Instance methods
    // ------------------------------------------------------------

    /** Clear and reset this <code>Session</code> so that
     *  no <code>Goal</code>s are considered satisfied.
     */
    public void clear()
    {
        this.satisfiedGoals.clear();
    }

    /** Add a satisfied <code>Goal</code> to this context.
     *
     *  @param goal The <code>Goal</code> that is satisfied.
     */
    void addSatisfiedGoal( Goal goal )
    {
        this.satisfiedGoals.add( goal );
    }

    /** Remove a satisfied <code>Goal</code> from this context.
     *
     *  @param goal The <code>Goal</code> that is no longer satisfied.
     */
    void removeSatisfiedGoal( Goal goal )
    {
        this.satisfiedGoals.remove( goal );
    }

    /** Test a <code>Goal</code> to determine if it has been
     *  satisfied within this context.
     *
     *  @param goal The <code>Goal</code> to test.
     *
     *  @return <code>true</code> if the <code>Goal</code> has been
     *          satisfied within this context, otherwise <code>false</code>.
     */
    boolean isGoalSatisfied( Goal goal )
    {
        return this.satisfiedGoals.contains( goal );
    }

    /** Set the attributes for this <code>Session</code>.
     * 
     *  @param attributes The attributes for this <code>Session</code>.
     */
    public void setAttributes( Map attributes )
    {
        this.attributes = attributes;
    }

    /** Get the attributes from this <code>Session</code>.
     * 
     *  @return Attributes for this <code>Session</code>.
     */
    public Map getAttributes()
    {
        return this.attributes;
    }

    /** Set an attribute for this <code>Session</code>.
     * 
     *  @param key Attribute key.
     *  @param value Attribute value.
     */
    public void setAttribute( String key, Object value )
    {
        getAttributes().put( key, value );
    }

    /** Get an attribute from this <code>Session</code>.
     * 
     *  @return An attribute from this <code>Session</code>.
     */
    public Object getAttribute( String key )
    {
        return getAttributes().get( key );
    }

    /** Log an informative message.
     *
     *  @param msg The message.
     */
    public void info( String msg )
    {
        // intentionally left blank
    }

    /** Log a warning message.
     *
     *  @param msg The message.
     */
    public void warn( String msg )
    {
        // intentionally left blank
    }

    /** Log an error message.
     *
     *  @param msg The message.
     */
    public void error( String msg )
    {
        // intentionally left blank
    }

}
