package org.apache.maven.project;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.File;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.maven.MavenUtils;

/**
 * Test the semantics of ids inside project.xml.
 *
 * @author Brett Porter <a href="mailto:brett@apache.org">brett@apache.org</a>
 * @version $Id: LegacyIdTest.java 517014 2007-03-11 21:15:50Z ltheussl $
 */
public class LegacyIdTest
    extends TestCase
{
    /**
     * Test for legacyToStandardId
     */
    public void testLegacyToStandardId()
    {
        // test 'ant' -> 'ant:ant'
        String legacyId = "ant";
        String standardId = "ant:ant";
        assertEquals( "Single id conversion failed", standardId, Project.legacyToStandardId( legacyId ) );

        // test 'ant:ant' unchanged
        legacyId = "ant:ant";
        assertEquals( "Standard id conversion failed", standardId, Project.legacyToStandardId( legacyId ) );

        // test 'ant+optional' -> ant:ant-optional
        legacyId = "ant+optional";
        standardId = "ant:ant-optional";
        assertEquals( "Plus format id conversion failed", standardId, Project.legacyToStandardId( legacyId ) );
    }

    /**
     * Test for legacyToDependencyKey
     */
    public void testLegacyToDependencyId()
    {
        Project project = new Project();
        addDependency( project, "ant", "ant", "jar" );
        addDependency( project, "ant", "ant-optional", "jar" );
        addDependency( project, "ant", "ant-windows", "dll" );

        // test 'ant' -> 'ant:ant:jar'
        String legacyId = "ant";
        String standardId = "ant:ant:jar";
        assertEquals( "Single id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant:ant' -> 'ant:ant:jar'
        legacyId = "ant:ant";
        assertEquals( "Standard id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant+optional' -> ant:ant-optional:jar
        legacyId = "ant+optional";
        standardId = "ant:ant-optional:jar";
        assertEquals( "Plus format id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant:ant:jar' unchanged
        legacyId = "ant:ant:jar";
        standardId = "ant:ant:jar";
        assertEquals( "Standard id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant:ant:tld' unchanged
        legacyId = "ant:ant:tld";
        standardId = "ant:ant:tld";
        assertEquals( "Standard id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant:ant-windows' gets correct one
        legacyId = "ant:ant-windows";
        standardId = "ant:ant-windows:dll";
        assertEquals( "Standard id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );

        // test 'ant:ant-windows:dll' unchanged
        legacyId = "ant:ant-windows:dll";
        standardId = "ant:ant-windows:dll";
        assertEquals( "Standard id conversion failed", standardId, project.legacyToDependencyKey( legacyId ) );
    }

    private static void addDependency( Project project, String groupId, String artifactId, String type )
    {
        Dependency dep = new Dependency();
        dep.setId( groupId + ":" + artifactId );
        dep.setGroupId( groupId );
        dep.setArtifactId( artifactId );
        dep.setType( type );
        dep.setVersion( "1.0-SNAPSHOT" );
        project.addDependency( dep );
    }

    public void testIdOnly()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/test-id-only.xml" ) );
        assertEquals( "check id", "artifactId:artifactId", p.getId() );
        assertEquals( "check artifact id", "artifactId", p.getArtifactId() );
        assertEquals( "check group id", "artifactId", p.getGroupId() );
        assertEquals( "check id", "artifactId:artifactId", p.getId() );
    }

    public void testGroupIdAndArtifactId()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/test-group-id-and-artifact-id.xml" ) );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
        assertEquals( "check artifact id", "artifactId", p.getArtifactId() );
        assertEquals( "check group id", "groupId", p.getGroupId() );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
    }

    public void testGroupIdAndIdAsArtifactId()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/test-group-id-and-id-as-artifact-id.xml" ) );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
        assertEquals( "check artifact id", "artifactId", p.getArtifactId() );
        assertEquals( "check group id", "groupId", p.getGroupId() );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
    }

    public void testIdAndGroupIdAndArtifactId()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/test-id-and-groupId-and-artifactId.xml" ) );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
        assertEquals( "check artifact id", "artifactId", p.getArtifactId() );
        assertEquals( "check group id", "groupId", p.getGroupId() );
        assertEquals( "check id", "groupId:artifactId", p.getId() );
    }

    public void testExtendNone()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/extend/test-none.xml" ) );
        assertEquals( "check group id", "base-groupId", p.getGroupId() );
    }

    public void testExtendIdOnly()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/extend/test-id-only.xml" ) );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "base-groupId", p.getGroupId() );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
    }

    public void testExtendOnlyArtifactIds()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/extend/test-artifact-id-only-extend.xml" ) );
        assertEquals( "check id", "childArtifactId:childArtifactId", p.getId() );
        assertEquals( "check artifact id", "childArtifactId", p.getArtifactId() );
        assertEquals( "check group id", "childArtifactId", p.getGroupId() );
        assertEquals( "check id", "childArtifactId:childArtifactId", p.getId() );
    }

    public void testExtendArtifactIdOnly()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/extend/test-artifact-id-only.xml" ) );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "base-groupId", p.getGroupId() );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
    }

    public void testExtendArtifactIdOnlyOverMatching()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/extend/test-artifact-id-only-over-matching.xml" ) );
        assertEquals( "check id", "base-id:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "base-id", p.getGroupId() );
        assertEquals( "check id", "base-id:extend-artifactId", p.getId() );
    }

    public void testExtendArtifactIdOnlyOverDifferent()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/extend/test-artifact-id-only-over-different.xml" ) );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "base-groupId", p.getGroupId() );
        assertEquals( "check id", "base-groupId:extend-artifactId", p.getId() );
    }

    public void testExtendIdOverrideGroupId()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/extend/test-id-override-groupid.xml" ) );
        assertEquals( "check id", "extend-artifactId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "extend-artifactId", p.getGroupId() );
        assertEquals( "check id", "extend-artifactId:extend-artifactId", p.getId() );
    }

    public void testExtendGroupIdAndArtifactId()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/extend/test-group-id-and-artifact-id.xml" ) );
        assertEquals( "check id", "extend-groupId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "extend-groupId", p.getGroupId() );
        assertEquals( "check id", "extend-groupId:extend-artifactId", p.getId() );
    }

    public void testExtendGroupIdAndIdAsArtifactId()
        throws Exception
    {
        Project p = MavenUtils
            .getProject( new File( System.getProperty( "basedir" ),
                                   "src/test/test-data/pom/extend/test-group-id-and-id-as-artifact-id.xml" ) );
        assertEquals( "check id", "extend-groupId:extend-artifactId", p.getId() );
        assertEquals( "check artifact id", "extend-artifactId", p.getArtifactId() );
        assertEquals( "check group id", "extend-groupId", p.getGroupId() );
        assertEquals( "check id", "extend-groupId:extend-artifactId", p.getId() );
    }

    public void testDependencies()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( System.getProperty( "basedir" ),
                                                     "src/test/test-data/pom/dependencies.xml" ) );
        assertEquals( "check id", "ant:ant", p.getDependency( "ant:ant" ).getId() );
        assertEquals( "check id with +", "ant:ant-optional", p.getDependency( "ant:ant-optional" ).getId() );
        assertEquals( "check id with -", "ant-optional:ant-optional", p.getDependency( "ant-optional:ant-optional" )
            .getId() );
        assertEquals( "check id with id and artifactId", "groupId:artifactId", p.getDependency( "groupId:artifactId" )
            .getId() );
    }

    /**
     * Generate the test suite.
     *
     * @return the test suite
     */
    public static Test suite()
    {
        return new TestSuite( LegacyIdTest.class );
    }
}
