package org.apache.maven.project;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.io.File;
import java.util.Arrays;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

import org.apache.maven.MavenUtils;
import org.codehaus.plexus.util.StringUtils;

public class ProjectInheritanceTest
    extends TestCase
{
    private String TEST_DOCUMENT = null;

    private String TEST_DOCUMENT2 = null;

    private String baseDir;

    private String FS = null;

    public ProjectInheritanceTest( String testName )
    {
        super( testName );
    }

    public static Test suite()
    {
        return new TestSuite( ProjectInheritanceTest.class );
    }

    protected void setUp()
        throws Exception
    {
        super.setUp();
        baseDir = System.getProperty( "basedir" );
        assertNotNull( "The system property basedir was not defined.", baseDir );
        String fs = System.getProperty( "file.separator" );
        assertNotNull( "The system property file.separator was not defined.", fs );
        FS = fs;
        TEST_DOCUMENT = baseDir + "/src/test/extend/project-child.xml";
        TEST_DOCUMENT2 = baseDir + "/src/test/extend/child_project/project-child2.xml";
    }

    public void testProjectMapping()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( TEST_DOCUMENT ) );

        // Make sure the groupId is inherited correctly.
        assertEquals( "groupId", p.getGroupId() );

        assertEquals( "Super Extendo", p.getName() );
        assertEquals( "groupId:super-extendo", p.getId() );
        assertEquals( "Apache Software Foundation", p.getOrganization().getName() );
        assertEquals( "http://www.apache.org/", p.getOrganization().getUrl() );

        //Test siteAddress / siteDirectory
        assertEquals( "theSiteAddress", p.getSiteAddress() );
        assertEquals( "theSiteDirectory", p.getSiteDirectory() );

        // Test Dependencies
        assertEquals( "child-dep1:child-dep1", ( (Dependency) p.getDependencies().get( 0 ) ).getId() );
        assertEquals( "1.0", ( (Dependency) p.getDependencies().get( 0 ) ).getVersion() );

        baseDir = new File( p.getContext().findVariable( "basedir" ).toString() ).getCanonicalPath();
        // Test Source Directories
        assertEquals( baseDir + FS + "src" + FS + "java", p.getBuild().getSourceDirectory() );
        // Unit Test Source Directories
        assertEquals( baseDir + FS + "src" + FS + "test" + FS + "java", p.getBuild().getUnitTestSourceDirectory() );

        // Test Aspect Source Directories
        //assertEquals("src/aspect", p.getBuild().getAspectSourceDirectory());

        String iuSrc = p.getBuild().getIntegrationUnitTestSourceDirectory();
        assertTrue( "Should be empty in maven's project descriptor: [" + iuSrc + "]", StringUtils.isEmpty( iuSrc ) );

        // Unit Test patterns
        List unitTestIncludes = p.getBuild().getUnitTest().getIncludes();
        List unitTestExcludes = p.getBuild().getUnitTest().getExcludes();

        assertEquals( "**/*Test*.java", (String) unitTestIncludes.get( 0 ) );
        assertEquals( "**/TestAll.java", (String) unitTestExcludes.get( 0 ) );
        assertEquals( "**/*Abstract*.java", (String) unitTestExcludes.get( 1 ) );

        SourceModification sm0 = (SourceModification) p.getBuild().getSourceModifications().get( 0 );
        assertEquals( "java.util.logging.Logger", sm0.getClassName() );
        assertEquals( "**/Jdk14Logger.java", (String) sm0.getExcludes().get( 0 ) );

        // Test Versions
        Version v0 = (Version) p.getVersions().get( 0 );
        assertEquals( "b1", v0.getId() );
        assertEquals( "1.0-b1", v0.getName() );
        assertEquals( "MAVEN_1_0_B1", v0.getTag() );

        // Test the version map
        Version v4 = p.getVersionById( "b4" );
        assertEquals( "b4", v4.getId() );
        assertEquals( "1.0-b4", v4.getName() );
        assertEquals( "MAVEN_1_0_B4", v4.getTag() );
    }

    public void testProjectMappingExtends()
        throws Exception
    {
        Project p = MavenUtils.getProject( new File( TEST_DOCUMENT2 ) );

        // Make sure the groupId is inherited correctly.
        assertEquals( "groupId", p.getGroupId() );

        assertEquals( "Child Project", p.getName() );
        assertEquals( "groupId:child", p.getId() );

        // Test organization inheritance.
        assertNotNull( p.getOrganization() );
        assertEquals( "Apache Software Foundation", p.getOrganization().getName() );
        assertEquals( "http://www.apache.org/", p.getOrganization().getUrl() );

        // Test siteAddress / siteDirectory
        assertEquals( "theSiteAddress", p.getSiteAddress() );
        assertEquals( "theSiteDirectory", p.getSiteDirectory() );

        boolean found = false;
        List resources = p.getBuild().getResources();
        for ( Iterator i = resources.iterator(); i.hasNext(); )
        {
            Resource r = (Resource) i.next();
            File dir = new File( p.getFile().getParent(), "src" + FS + "messages" ).getCanonicalFile();
            if ( r.getDirectory().equals( dir.getPath() ) )
            {
                assertEquals( "check target path", "org/apache/maven/messages", r.getTargetPath() );
                assertEquals( "check includes", Arrays.asList( new String[] { "messages*.properties" } ), r
                    .getIncludes() );
                assertEquals( "check excludes", Collections.EMPTY_LIST, r.getExcludes() );
                found = true;
            }
        }
        assertTrue( "Check found resources for src/messages", found );
    }

}
