package org.apache.maven.werkz;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.util.List;

import junit.framework.TestCase;

public class GoalTest
    extends TestCase
{
    private Session session;

    public GoalTest( String name )
    {
        super( name );
    }

    public void setUp()
    {
        this.session = new Session();
    }

    public void tearDown()
    {
        this.session = null;
    }

    public void testHashCode()
    {

        Goal a1 = new Goal( "a" );
        Goal a2 = new Goal( "a" );

        Goal b1 = new Goal( "b" );
        Goal b2 = new Goal( "b" );

        assertEquals( a1.hashCode(), a2.hashCode() );

        assertEquals( b1.hashCode(), b2.hashCode() );

        assertTrue( a1.hashCode() != b1.hashCode() );
    }

    public void testEquals()
    {
        Goal a1 = new Goal( "a" );
        Goal a2 = new Goal( "a" );

        Goal b1 = new Goal( "b" );
        Goal b2 = new Goal( "b" );

        assertEquals( a1, a2 );

        assertEquals( b1, b2 );

        assertTrue( !a1.equals( b1 ) );
    }

    public void testHasBeenMet()
    {
        Goal g = new Goal( "a" );

        assertTrue( !g.isSatisfied( this.session ) );

        this.session.addSatisfiedGoal( g );

        assertTrue( g.isSatisfied( this.session ) );

        this.session.removeSatisfiedGoal( g );

        assertTrue( !g.isSatisfied( this.session ) );
    }

    public void testAddPrerequisite_NoCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );

        try
        {
            g1.addPrecursor( g2 );
            g1.addPrecursor( g3 );

            assertEquals( 2, g1.getPrecursors().size() );

            assertTrue( g1.getPrecursors().contains( g2 ) );
            assertTrue( g1.getPrecursors().contains( g3 ) );

            assertEquals( 0, g2.getPrecursors().size() );
            assertEquals( 0, g3.getPrecursors().size() );

        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPostcursor_NoCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );

        try
        {
            g1.addPostcursor( g2 );
            g3.addPrecursor( g2 );
            g3.addPostcursor( g4 );
            g4.addPostcursor( g5 );
            g4.addPostcursor( g6 );

            assertEquals( 0, g1.getPrecursors().size() );
            assertEquals( 1, g1.getPostcursors().size() );
            assertEquals( 1, g2.getPrecursors().size() );
            assertEquals( 1, g2.getPostcursors().size() );
            assertEquals( 1, g3.getPrecursors().size() );
            assertEquals( 1, g3.getPostcursors().size() );
            assertEquals( 1, g4.getPrecursors().size() );
            assertEquals( 2, g4.getPostcursors().size() );
            assertEquals( 1, g5.getPrecursors().size() );
            assertEquals( 0, g5.getPostcursors().size() );
            assertEquals( 1, g6.getPrecursors().size() );
            assertEquals( 0, g6.getPostcursors().size() );

            assertTrue( g1.getPostcursors().contains( g2 ) );
            assertTrue( g4.getPostcursors().contains( g5 ) );
            assertTrue( g4.getPostcursors().contains( g6 ) );
            assertTrue( g5.getPrecursors().contains( g4 ) );
            assertTrue( g6.getPrecursors().contains( g4 ) );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPrecursor_LocalCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );

        try
        {
            g1.addPrecursor( g2 );

            try
            {
                g2.addPrecursor( g1 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testPostcursor_LocalCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );

        try
        {
            g1.addPostcursor( g2 );

            try
            {
                g2.addPostcursor( g1 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPrecursor_DistantFullCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );
            g3.addPrecursor( g4 );
            g4.addPrecursor( g5 );
            g5.addPrecursor( g6 );
            g6.addPrecursor( g7 );
            g7.addPrecursor( g8 );
            g8.addPrecursor( g9 );

            try
            {
                g9.addPrecursor( g1 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPostcursor_DistantFullCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );
            g3.addPostcursor( g4 );
            g4.addPostcursor( g5 );
            g5.addPostcursor( g6 );
            g6.addPostcursor( g7 );
            g7.addPostcursor( g8 );
            g8.addPostcursor( g9 );

            try
            {
                g9.addPostcursor( g1 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPrecursor_DistantPartialCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );
            g3.addPrecursor( g4 );
            g4.addPrecursor( g5 );
            g5.addPrecursor( g6 );
            g6.addPrecursor( g7 );
            g7.addPrecursor( g8 );
            g8.addPrecursor( g9 );

            try
            {
                g9.addPrecursor( g5 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPostcursor_DistantPartialCycle()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );
            g3.addPostcursor( g4 );
            g4.addPostcursor( g5 );
            g5.addPostcursor( g6 );
            g6.addPostcursor( g7 );
            g7.addPostcursor( g8 );
            g8.addPostcursor( g9 );

            try
            {
                g9.addPostcursor( g5 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPrecursor_DistantPartialCycle_PieceWise()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );
            g3.addPrecursor( g4 );
            //g4.addPrecursor( g5 );
            g5.addPrecursor( g6 );
            g6.addPrecursor( g7 );
            g7.addPrecursor( g8 );
            g8.addPrecursor( g9 );
            g9.addPrecursor( g4 );

            try
            {
                g4.addPrecursor( g5 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAddPostcursor_DistantPartialCycle_PieceWise()
    {
        Goal g1 = new Goal( "goal-1" );
        Goal g2 = new Goal( "goal-2" );
        Goal g3 = new Goal( "goal-3" );
        Goal g4 = new Goal( "goal-4" );
        Goal g5 = new Goal( "goal-5" );
        Goal g6 = new Goal( "goal-6" );
        Goal g7 = new Goal( "goal-7" );
        Goal g8 = new Goal( "goal-8" );
        Goal g9 = new Goal( "goal-9" );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );
            g3.addPostcursor( g4 );
            //g4.addPostcursor( g5 );
            g5.addPostcursor( g6 );
            g6.addPostcursor( g7 );
            g7.addPostcursor( g8 );
            g8.addPostcursor( g9 );
            g9.addPostcursor( g4 );

            try
            {
                g4.addPostcursor( g5 );

                fail( "Should have thrown CyclicGoalChainException" );
            }
            catch ( CyclicGoalChainException e )
            {
                // expected and correct
            }
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAttain_Linear()
    {
        GoalTracker tracker = new GoalTracker();

        Goal g1 = new MockGoal( "goal-1", tracker );

        Goal g2 = new MockGoal( "goal-2", tracker );

        Goal g3 = new MockGoal( "goal-3", tracker );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );

            tracker.addExpectedGoal( g3 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g1 );

            g1.attain( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testPostAttain_Linear()
    {
        GoalTracker tracker = new GoalTracker();

        Goal g1 = new MockGoal( "goal-1", tracker );

        Goal g2 = new MockGoal( "goal-2", tracker );

        Goal g3 = new MockGoal( "goal-3", tracker );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );

            tracker.addExpectedGoal( g1 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g3 );

            g3.attain( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testPostPercolate_Linear()
    {
        GoalTracker tracker = new GoalTracker();

        Goal g1 = new MockGoal( "goal-1", tracker );

        Goal g2 = new MockGoal( "goal-2", tracker );

        Goal g3 = new MockGoal( "goal-3", tracker );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );

            tracker.addExpectedGoal( g1 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g3 );

            g1.percolate( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testAttain_Tree()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1 = new MockGoal( "goal-1", tracker );

        MockGoal g2 = new MockGoal( "goal-2", tracker );

        MockGoal g3 = new MockGoal( "goal-3", tracker );

        MockGoal g4 = new MockGoal( "goal-4", tracker );

        MockGoal g5 = new MockGoal( "goal-5", tracker );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );
            g2.addPrecursor( g4 );
            g4.addPrecursor( g5 );

            tracker.addExpectedGoal( g3 );
            tracker.addExpectedGoal( g5 );
            tracker.addExpectedGoal( g4 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g1 );

            g1.attain( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testPostPercolate_Tree()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1 = new MockGoal( "goal-1", tracker );

        MockGoal g2 = new MockGoal( "goal-2", tracker );

        MockGoal g3 = new MockGoal( "goal-3", tracker );

        MockGoal g4 = new MockGoal( "goal-4", tracker );

        MockGoal g5 = new MockGoal( "goal-5", tracker );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );
            g2.addPostcursor( g4 );
            g4.addPostcursor( g5 );

            tracker.addExpectedGoal( g1 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g3 );
            tracker.addExpectedGoal( g4 );
            tracker.addExpectedGoal( g5 );

            g1.percolate( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testAttain_Tree_PartiallyNotRequired()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1 = new MockGoal( "goal-1", tracker );

        MockGoal g2 = new MockGoal( "goal-2", tracker );

        MockGoal g3 = new MockGoal( "goal-3", tracker );

        MockGoal g4 = new MockGoal( "goal-4", tracker );

        MockGoal g5 = new MockGoal( "goal-5", tracker );

        try
        {
            g1.addPrecursor( g2 );
            g2.addPrecursor( g3 );
            g2.addPrecursor( g4 );
            g4.addPrecursor( g5 );

            g4.requiresAction( false );
            g1.requiresAction( false );

            tracker.addExpectedGoal( g3 );
            tracker.addExpectedGoal( g5 );
            // tracker.addExpectedGoal( g4 );
            tracker.addExpectedGoal( g2 );
            //tracker.addExpectedGoal( g1 );

            g1.attain( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testPercolate_Tree_PartiallyNotRequired()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1 = new MockGoal( "goal-1", tracker );

        MockGoal g2 = new MockGoal( "goal-2", tracker );

        MockGoal g3 = new MockGoal( "goal-3", tracker );

        MockGoal g4 = new MockGoal( "goal-4", tracker );

        MockGoal g5 = new MockGoal( "goal-5", tracker );

        try
        {
            g1.addPostcursor( g2 );
            g2.addPostcursor( g3 );
            g2.addPostcursor( g4 );
            g4.addPostcursor( g5 );

            g4.requiresAction( false );
            g1.requiresAction( false );

            //tracker.addExpectedGoal( g1 );
            tracker.addExpectedGoal( g2 );
            tracker.addExpectedGoal( g3 );
            //tracker.addExpectedGoal( g4 );
            tracker.addExpectedGoal( g5 );

            g1.percolate( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }

        tracker.verify();
    }

    public void testAttain_Tree_Overlap()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1_1 = new MockGoal( "goal-1-1", tracker );

        MockGoal g1_2 = new MockGoal( "goal-1-2", tracker );

        MockGoal g1_3 = new MockGoal( "goal-1-3", tracker );

        MockGoal g2_1 = new MockGoal( "goal-2-1", tracker );

        MockGoal g2_2 = new MockGoal( "goal-2-2", tracker );

        MockGoal g2_3 = new MockGoal( "goal-2-3", tracker );

        MockGoal gc_1 = new MockGoal( "goal-c-1", tracker );

        MockGoal gc_2 = new MockGoal( "goal-c-2", tracker );

        try
        {
            g1_1.addPrecursor( g1_2 );
            g1_2.addPrecursor( g1_3 );

            g2_1.addPrecursor( g2_2 );
            g2_2.addPrecursor( g2_3 );

            g1_3.addPrecursor( gc_1 );
            g2_3.addPrecursor( gc_1 );

            gc_1.addPrecursor( gc_2 );

            // ----------------------------------------
            //   test 1
            // ----------------------------------------

            tracker.clear();

            tracker.addExpectedGoal( gc_2 );
            tracker.addExpectedGoal( gc_1 );
            tracker.addExpectedGoal( g1_3 );
            tracker.addExpectedGoal( g1_2 );
            tracker.addExpectedGoal( g1_1 );

            g1_1.attain( this.session );

            tracker.verify();

            // ----------------------------------------
            //   test 2
            // ----------------------------------------

            tracker.clear();

            tracker.addExpectedGoal( g2_3 );
            tracker.addExpectedGoal( g2_2 );
            tracker.addExpectedGoal( g2_1 );

            g2_1.attain( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testPercolate_Tree_Overlap()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g1_1 = new MockGoal( "goal-1-1", tracker );

        MockGoal g1_2 = new MockGoal( "goal-1-2", tracker );

        MockGoal g1_3 = new MockGoal( "goal-1-3", tracker );

        MockGoal g2_1 = new MockGoal( "goal-2-1", tracker );

        MockGoal g2_2 = new MockGoal( "goal-2-2", tracker );

        MockGoal g2_3 = new MockGoal( "goal-2-3", tracker );

        MockGoal gc_1 = new MockGoal( "goal-c-1", tracker );

        MockGoal gc_2 = new MockGoal( "goal-c-2", tracker );

        try
        {
            g1_1.addPostcursor( g1_2 );
            g1_2.addPostcursor( g1_3 );

            g2_1.addPostcursor( g2_2 );
            g2_2.addPostcursor( g2_3 );

            g1_3.addPostcursor( gc_1 );
            g2_3.addPostcursor( gc_1 );

            gc_1.addPostcursor( gc_2 );

            // ----------------------------------------
            //   test 1
            // ----------------------------------------

            tracker.clear();

            tracker.addExpectedGoal( g1_1 );
            tracker.addExpectedGoal( g1_2 );
            tracker.addExpectedGoal( g1_3 );
            tracker.addExpectedGoal( gc_1 );
            tracker.addExpectedGoal( gc_2 );

            g1_1.percolate( this.session );

            tracker.verify();

            // ----------------------------------------
            //   test 2
            // ----------------------------------------

            tracker.clear();

            tracker.addExpectedGoal( g2_1 );
            tracker.addExpectedGoal( g2_2 );
            tracker.addExpectedGoal( g2_3 );

            g2_1.percolate( this.session );
        }
        catch ( CyclicGoalChainException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testRequiresAction_False()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g = new MockGoal( "goal-1", tracker );

        g.requiresAction( false );

        try
        {
            g.attain( this.session );

            tracker.verify();
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testRequiresAction_True()
    {
        GoalTracker tracker = new GoalTracker();

        MockGoal g = new MockGoal( "goal-1", tracker );

        tracker.addExpectedGoal( g );

        try
        {
            g.attain( this.session );

            tracker.verify();
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testPreGoalCallback_AddRemove()
    {
        MockCallback callback = new MockCallback();

        Goal g = new Goal( "goal-1" );

        g.addPreGoalCallback( callback );
        g.addPreGoalCallback( callback );

        List callbacks = g.getPreGoalCallbacks();

        assertEquals( 2, callbacks.size() );

        assertSame( callback, callbacks.get( 0 ) );

        assertSame( callback, callbacks.get( 1 ) );

        g.removePreGoalCallback( callback );

        callbacks = g.getPreGoalCallbacks();

        assertEquals( 0, callbacks.size() );
    }

    public void testPostGoalCallback_AddRemove()
    {
        MockCallback callback = new MockCallback();

        Goal g = new Goal( "goal-1" );

        g.addPostGoalCallback( callback );
        g.addPostGoalCallback( callback );

        List callbacks = g.getPostGoalCallbacks();

        assertEquals( 2, callbacks.size() );

        assertSame( callback, callbacks.get( 0 ) );

        assertSame( callback, callbacks.get( 1 ) );

        g.removePostGoalCallback( callback );

        callbacks = g.getPostGoalCallbacks();

        assertEquals( 0, callbacks.size() );
    }

    public void testPreActionCallback_AddRemove()
    {
        MockCallback callback = new MockCallback();

        Goal g = new Goal( "goal-1" );

        g.addPreActionCallback( callback );
        g.addPreActionCallback( callback );

        List callbacks = g.getPreActionCallbacks();

        assertEquals( 2, callbacks.size() );

        assertSame( callback, callbacks.get( 0 ) );

        assertSame( callback, callbacks.get( 1 ) );

        g.removePreActionCallback( callback );

        callbacks = g.getPreActionCallbacks();

        assertEquals( 0, callbacks.size() );
    }

    public void testPostActionCallback_AddRemove()
    {
        MockCallback callback = new MockCallback();

        Goal g = new Goal( "goal-1" );

        g.addPostActionCallback( callback );
        g.addPostActionCallback( callback );

        List callbacks = g.getPostActionCallbacks();

        assertEquals( 2, callbacks.size() );

        assertSame( callback, callbacks.get( 0 ) );

        assertSame( callback, callbacks.get( 1 ) );

        g.removePostActionCallback( callback );

        callbacks = g.getPostActionCallbacks();

        assertEquals( 0, callbacks.size() );
    }
}
