package org.apache.maven.werkz;

/* ====================================================================
 *   Licensed to the Apache Software Foundation (ASF) under one or more
 *   contributor license agreements.  See the NOTICE file distributed with
 *   this work for additional information regarding copyright ownership.
 *   The ASF licenses this file to You under the Apache License, Version 2.0
 *   (the "License"); you may not use this file except in compliance with
 *   the License.  You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 * ====================================================================
 */

import java.util.Arrays;

import junit.framework.TestCase;

public class ProjectTest
    extends TestCase
{
    private Session session;

    public ProjectTest( String name )
    {
        super( name );
    }

    public void setUp()
    {
        this.session = new Session();
    }

    public void tearDown()
    {
        this.session = null;
    }

    public void testAddGoal()
    {
        Goal g = new Goal( "goal-1" );

        WerkzProject project = new WerkzProject();

        project.addGoal( g );

        assertSame( g, project.getGoal( "goal-1" ) );
    }

    public void testGetGoal_Bogus()
    {
        WerkzProject project = new WerkzProject();

        assertNull( project.getGoal( "goal-1" ) );
    }

    public void testAttainGoal()
    {
        GoalTracker tracker = new GoalTracker();

        WerkzProject project = new WerkzProject();

        MockGoal g = new MockGoal( "goal-1", tracker );

        project.addGoal( g );

        tracker.addExpectedGoal( g );

        try
        {
            project.attainGoal( "goal-1", this.session );

            tracker.verify();
        }
        catch ( NoSuchGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testAttainGoal_Bogus()
    {
        WerkzProject project = new WerkzProject();

        try
        {
            project.attainGoal( "goal-1", this.session );

            fail( "Should have thrown NoSuchGoalException" );
        }
        catch ( NoSuchGoalException e )
        {
            // expected and correct
        }
        catch ( UnattainableGoalException e )
        {
            fail( e.getLocalizedMessage() );
        }
        catch ( NoActionDefinitionException e )
        {
            fail( e.getLocalizedMessage() );
        }
    }

    public void testGetExecutionChain()
        throws Exception
    {
        WerkzProject project = new WerkzProject();

        Goal a = new Goal( "a" );
        Goal b = new Goal( "b" );
        Goal c = new Goal( "c" );
        Goal d = new Goal( "d" );
        Goal e = new Goal( "e" );
        Goal f = new Goal( "f" );

        a.addPrecursor( b );
        a.addPrecursor( c );

        b.addPrecursor( d );

        c.addPrecursor( e );

        d.addPrecursor( f );
        e.addPrecursor( f );

        project.addGoal( a );
        project.addGoal( b );
        project.addGoal( c );
        project.addGoal( d );
        project.addGoal( e );
        project.addGoal( f );

        Goal[] chain = project.getExecutionChain( "a" );

        assertEquals( 6, chain.length );

        assertSame( a, chain[5] );

        // ----------
        //     a
        // ----------

        assertBefore( b, a, chain );
        assertBefore( c, a, chain );
        assertBefore( d, a, chain );
        assertBefore( e, a, chain );
        assertBefore( f, a, chain );

        // ----------
        //     b
        // ----------

        assertBefore( d, b, chain );
        assertBefore( f, b, chain );

        // ----------
        //     c
        // ----------

        assertBefore( e, c, chain );
        assertBefore( f, c, chain );

        // ----------
        //     d
        // ----------

        assertBefore( f, d, chain );

        // ----------
        //     e
        // ----------

        assertBefore( f, e, chain );

        // 

        assertContains( a, chain );
        assertContains( b, chain );
        assertContains( c, chain );
        assertContains( d, chain );
        assertContains( e, chain );
    }

    protected void assertBefore( Goal first, Goal after, Goal[] chain )
    {
        int firstIndex = -1;
        int afterIndex = -1;

        for ( int i = 0; i < chain.length; ++i )
        {
            if ( chain[i].equals( first ) )
            {
                firstIndex = i;
            }
            else if ( chain[i].equals( after ) )
            {
                afterIndex = i;
            }
        }

        if ( firstIndex < 0 )
        {
            fail( "first element " + first + " not in " + Arrays.asList( chain ) );
        }

        if ( firstIndex < 0 )
        {
            fail( "after element " + after + " not in " + Arrays.asList( chain ) );
        }

        assertTrue( firstIndex < afterIndex );
    }

    protected void assertContains( Object obj, Object[] array )
    {
        for ( int i = 0; i < array.length; ++i )
        {
            if ( array[i].equals( obj ) )
            {
                return;
            }
        }

        fail( obj + " not in " + Arrays.asList( array ) );
    }
}
